--| 10/10/2012
--| 
--| ENUNCIADO:
--| Utilizando memoria compartida, desarrolla un paquete en Ada, y un
--| procedimiento para probarlo, que multiplique dos matrices de reales
--| de cualquier tamao, pero buscando la forma de que haya varios
--| procesos realizando la multiplicacin, para que en caso de ejecutarlo
--| en una arquitectura paralela se obtenga un mejor rendimiento.
--| 
--| Desarrolla tambin un paquete que haga la multiplicacin de forma
--| secuencial. Sobre una arquitectura monoprocesador, mide desde el
--| programa lo que cuesta ejecutar cada caso y entrega esas mediciones,
--| junto con tus conclusiones. No olvides indicar las prestaciones del
--| equipo en el que lo has ejecutado y la carga que soportaba el
--| procesador antes de hacerlo (y haz ambas mediciones con una misma
--| carga).
--| El tamao de los datos ha de ser suficientemente grande para
--| que los resultados obtenidos sean significativos. Ejectalo varias
--| veces para diferentes tamaos de matriz para asegurarte de que la
--| cantidad de datos obtenida es suficiente para ser analizada.

--| 7. (opcional) Ejecuta las dos posibilidades del ejercicio anterior
--| sobre una arquitectura paralela, tomando tiempos y extrayendo tus
--| conclusiones.


--| CONCLUSION:
--| A falta de aadir el paralelismo con menos procesos, y no por cada celda de la matriz resultante.
--| Se puede observar como para trabajar con dos matrices de 160.000 numeros (de dos digitos)
--| cada una, al programa creado con paralelismo le cuesta mas tiempo calcular los numeros
--| cuando realmente deberia de ser al reves pues al haber mas procesos las cuentas a
--| realizar se terminan antes.
--| 
--| Sin embargo se intuye que esto es debido a la gran cantidad de procesos que se estan creando
--| para operaciones tan "simples" y/o por la forma de implementar internamente los procesos con ADA.
--| 
--| El equipo con el que se han realizado las pruebas cuenta con un Intel Core 2 Duo E8200 a 2.67Ghz
--| (2 nucleos), 4GiB de RAM DDR2 y Windows7 64bits.
--| 	
--| En el caso de la ejecucion del ejercicio secuencial "ada1x06_A - Secuencial" el uso de RAM y carga
--| del procesador ha sido de 20% y 2,5 MiB respectivamente. El tiempo de ejecucin ha sido
--| de 800 milisegundos (sin contar la creacion de los numeros, carga en RAM, ni su impresion en pantalla
--| que aumenta el tiempo total a 22 segundos)
--| 	
--| Con el ejercicio con paralelismo "ada1x06_B - Concurrente", es necesario el 56% de procesador
--| y 351 MiB de memoria RAM. Adems el tiempo asciende a 19 segundos (o 43 segundos de tiempo total).


--| INFO:
--| http://es.wikipedia.org/wiki/Multiplicacin_de_matrices
--| http://www.bluebit.gr/matrix-calculator/multiply.aspx
--| http://www.gedlc.ulpgc.es/docencia/NGA/paquetes.html
--| http://www.gedlc.ulpgc.es/docencia/mp_i/GuiaAda/ada12.html
--| http://www.williammartin.com/archives/resume/ada/index.html
--| http://www.adapower.com/index.php?Command=Class&ClassID=Utilities&CID=386



with Ada.Text_IO;
with Ada.Integer_Text_IO;
with matrices_con_tareas;
with Ada.Real_Time;
 
 

procedure ada1x06 is

	use Ada.Text_IO;
	use Ada.Integer_Text_IO;
	use Ada.Real_Time;
	
	
	-- Constantes para definir los tamaos de las matrices:
	-- Dos matrices de 350*350 = 245.000 numeros.
	filA : constant positive := 350;
	colA : constant positive := 350;
	colB : constant positive := 350;
	
	
	-- Instanciacion del paquete con las medidas de la matriz resultante:
	package m is new matrices_con_tareas( filas=>filA , columnas=>colB );
	use m;
	
	-- Paquete para la conversion de tipos (Duration,Time > Integer)
	package d is new Fixed_IO(Duration);
	use d;


	-- Se crea tres matrices del tipo matriz que esta en el paquete:
	mA   : Matriz( 1..filA, 1..colA );
	mB   : Matriz( 1..colA, 1..colB );
	mRes : Matriz( 1..filA, 1..colB );
	
	-- Dos variables para sacar los tiempos:
	t1,t2:Time;
	
begin

	-- Se crean los numeros aleatorios en el fichero "numeros.txt"
	-- despues se cargan en las matrices:
	createFileOfRandomNumbers( filA , colA , colB , numMax=>10 );
	readFileOfRandomNumbers( mA , mB );
		
		
	-- Se multiplican las matrices por una sobrecarga del operador.
	-- Tambien se toma el tiempo de inicio y el de fin:
	put_line("Calculando el resultado...");
	t1 := clock;
	mRes := mA * mB;
	t2 := clock;
	
	
	-- Se imprime el tiempo:
	new_line;
	put("Tiempo de calculo de la matriz: ");
	put( To_Duration(t2-t1));
	new_line;
	
	
	-- Y por ultimo se imprimen los datos de la matriz:
	put_line("Ahora se procedera a la impresion en 5 segundos: ");
	delay(5.0);	
	put( mRes );
	
end ada1x06;
