--| 15/03/2012
--| IMPLEMENTACIÓN DE LA COLA DINÁMICA


-- El siguiente paquete permite borrar la memoria dinámica.
with unchecked_deallocation;


-- Funciones del paquete:
package body cola_dinamica is



	--| INICIALIZA UNA COLA:
	--| Precondición: Se ha de recibir un parámetro de tipo ptCola (puntero a colas)
	--| Poscondición: Se hace que el puntero apunte a una nueva Cola y se inicializa su contenido a 0/null.
	procedure inicializar( c:in out ptCola ) is
	begin
		c          := new cola;
		c.numNodos := 0;
		c.primero  := null;
		c.ultimo   := null;
	end inicializar;



	--| INDICA SI LA COLA ESTA VACÍA:
	--| Precondición: Se ha de recibir un parámetro de tipo ptCola.
	--| Poscondición: Se devuelve un true si la cola esta vacía o un false en caso contrario.
	function esVacia( c:in ptCola ) return boolean is
	begin
		if c.numNodos > 0 then
			return false;
		else
			return true;
		end if;
	end esVacia;



	--| INSERTA UN NUEVO NODO AL FINAL DE LA COLA:
	--| Precondición: Se ha de recibir un parámetro de tipo ptCola, y el tipo de dato a guardar en los Nodos de la Cola.
	--| Poscondición: Se crea un nuevo Nodo y se agrega al final de la Cola con los nuevos datos.
	procedure insertar( c:in out ptCola; d:in datos ) is
		temp:ptNodo := new nodo;
	begin
		temp.elemento  := d;
		temp.siguiente := null;

		if c.numNodos = 0 then
			c.primero := temp;
		else
			c.ultimo.siguiente := temp;
		end if;

		c.ultimo  := temp;
		c.numNodos := c.numNodos + 1;
	end insertar;



	--| LEE LOS DATOS DEL PRIMER NODO:
	--| Precondición: Se ha de recibir un parámetro de tipo ptCola. Se ha de comprobar que no este vacía con la funcion esVacia()
	--| Poscondición: Se devuelven los datos del primer Nodo de la Cola.
	function leer( c:in ptCola ) return datos is
	begin
		return c.primero.elemento;
	end leer;



	--| EXTRAE LOS DATOS DEL PRIMER NODO:
	--| Precondición: Se ha de recibir un parámetro de tipo ptCola, y el tipo de dato a guardar en los Nodos de la Cola. Se ha de comprobar que no este vacía con la función esVacia()
	--| Poscondición: Devuelve por referencia los datos del primer Nodo y a continuacion lo libera.
	procedure extraer( c:in out ptCola; d:out datos ) is
		procedure borrar is new unchecked_deallocation(nodo,ptNodo);
		temp:ptNodo;
	begin
		d             := c.primero.elemento;
		temp          := c.primero;
		c.primero     := c.primero.siguiente;
		c.numNodos    := c.numNodos - 1;
		borrar(temp);
	end extraer;



	--| BORRA TODOS LOS NODOS DE LA COLA:
	--| Precondición: Se ha de recibir un parámetro de tipo ptCola.
	--| Poscondición: Borra recursivamente todos los Nodos y por ultimo la Cola.
	procedure borrar( c:in out ptCola ) is
		procedure borrar is new unchecked_deallocation(nodo,ptNodo);
		procedure borrar is new unchecked_deallocation(cola,ptCola);
		temp:ptNodo;
	begin
		if c.primero /= null then
			temp           := c.primero;
			c.primero      := c.primero.siguiente;
			borrar(c);     -- Envía el segundo Nodo para su borrado.
			borrar(temp);  -- Borra el primer Nodo.
		else
			borrar(c);     -- Borra la Cola.
		end if;
	end borrar;


end cola_dinamica;